// src/jump_to_dialog.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "jump_to_dialog.hh"
#include <sgpemv2/templates/sequences.tcc>
#include <sgpemv2/history.hh>
#include <sgpemv2/environment.hh>
#include <sgpemv2/simulation.hh>
#include <sgpemv2/resource.hh>

#include <iostream>
#include <sstream>

#include "gettext.h"

#include <cassert>

using namespace sgpem;
using namespace Gtk;
using namespace Glib;
using Gnome::Glade::Xml;
using std::vector;

JumpToDialog::JumpToDialog(BaseObjectType* cobject, const RefPtr<Xml>& glade) :
  Dialog(cobject), _glade(glade), _target_instant(0)
{
  _glade->get_widget("OK.Button", _ok_button);
  _glade->get_widget("Jump.Button", _jump_button);
  _glade->get_widget("Stop.Button", _stop_button);
  _glade->get_widget("Instant.Spin", _instant_spin);
  _glade->get_widget("ProgressBar", _progress);


  /** ATTACH SIGNAL HANDLERS FOR BUTTONS **/
  
  _jump_button->signal_clicked().connect(
      sigc::mem_fun(*this, &JumpToDialog::_on_jump));

  _stop_button->signal_clicked().connect(
      sigc::mem_fun(*this, &JumpToDialog::_on_stop));

  // FIXME: not implemented
  _stop_button->set_sensitive(false);
}

void
JumpToDialog::_on_jump()
{
  _ok_button->set_sensitive(false);
//  _stop_button->set_sensitive(true);
  _jump_button->set_sensitive(false);
  _progress->set_fraction(0.0);
  
  _target_instant = _instant_spin->get_value_as_int();

  Simulation::get_instance().attach(*this);
  Simulation::get_instance().jump_to(_target_instant);
  Simulation::get_instance().detach(*this);

  _ok_button->set_sensitive(true);
//  _stop_button->set_sensitive(false);
  _jump_button->set_sensitive(true);
}

void
JumpToDialog::_on_stop()
{
}

void
JumpToDialog::update(const Simulation& changed_simulation)
{
  unsigned int front = changed_simulation.get_history().get_front();

  double percent = static_cast<double>(front) / _target_instant;

  _progress->set_fraction(percent);
}
